library own_navbar;

import 'dart:math';

import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

enum IndicatorTypes {
  ULTRASMALL,
  SMALL,
  MEDIUM,
  LARGE,
  MAX,
}

class BottomNavbar extends StatefulWidget {
  List<String>? navItems;
  final double height;
  double? indicatorHeight;
  IndicatorTypes indicatorType;
  int animationDuration;
  Color? indicatorColor;
  TextStyle? textStyle;
  List<IconData>? navItems2;
  Color passiveColor = Colors.black54;
  Color activeColor = Colors.black;
  Cubic curve;
  Function(int) itemClicked;

  bool equalDuration;

  BottomNavbar({
    Key? key,
    this.navItems,
    required this.itemClicked,
    this.animationDuration = 100,
    this.equalDuration = true,
    this.indicatorColor,
    this.navItems2,
    this.indicatorHeight,
    this.textStyle,
    this.activeColor = Colors.black,
    this.passiveColor = Colors.black54,
    this.curve = Curves.easeOut,
    this.indicatorType = IndicatorTypes.MAX,
    required this.height,
  }) : super(key: key);

  @override
  _BottomNavbarState createState() => _BottomNavbarState();
}

class _BottomNavbarState extends State<BottomNavbar> {
  int initialIndex = 0;
  int initialDuration = 0;

  @override
  void initState() {
    initialDuration = widget.animationDuration;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: widget.height,
      child: Stack(
        children: [
          AnimatedContainer(
            alignment: Alignment(xCoordinate(), -1),
            duration: Duration(milliseconds: initialDuration),
            curve: widget.curve,
            child: Container(
              color: Colors.transparent,
              width: indicatorWidth(),
              alignment: Alignment(0, -1),
              child: Container(
                width: indicatorWidth() / indicatorSizeFactor(),
                height: widget.indicatorHeight ?? widget.height / 10,
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(indicatorWidth()),
                  color: colorReturner(),
                ),
              ),
            ),
          ),
          Container(
            alignment: Alignment.center,
            margin: EdgeInsets.only(
                top: widget.indicatorHeight ?? widget.height / 10),
            child: Row(
              children: [
                for (int i = 0; i < lengthReturner(); i++) ...{
                  Expanded(
                    child: InkWell(
                      splashColor: Colors.transparent,
                      highlightColor: Colors.transparent,
                      onTap: () {
                        if (initialIndex == i) {
                          return;
                        }
                        if (widget.equalDuration) {
                          initialDuration = ((initialIndex - i).abs() *
                              widget.animationDuration)
                              .round();
                        }

                        setState(() {
                          initialIndex = i;
                        });
                        widget.itemClicked.call(initialIndex);
                      },
                      child: Center(
                        child: NavItem(i),
                      ),
                    ),
                  ),
                },
              ],
            ),
          )
        ],
      ),
    );
  }

  xCoordinate() {
    if (widget.navItems == null && widget.navItems2 == null) {
      throw Exception(
        "At least one of navigation bar type must be defined.\nUse navItems or navItems2 or both",
      );
    }
    double k = 1;
    if (widget.navItems != null) {
      k = 2 / (widget.navItems!.length - 1);
    } else if (widget.navItems2 != null) {
      k = 2 / (widget.navItems2!.length - 1);
    }
    double y = (k * initialIndex) - 1;

    return y;
  }

  Color colorReturner() {
    if (widget.indicatorColor != null) {
      return widget.indicatorColor!;
    }
    return Color.fromRGBO(
        Random().nextInt(255), Random().nextInt(255), Random().nextInt(255), 1);
  }

  double indicatorWidth() {
    if (widget.navItems != null) {
      return MediaQuery.of(context).size.width / widget.navItems!.length;
    } else if (widget.navItems2 != null) {
      return MediaQuery.of(context).size.width / widget.navItems2!.length;
    } else {
      return 10;
    }
  }

  Widget NavItem(int i) {
    if (widget.navItems != null && widget.navItems2 != null) {
      return Column(
        children: [
          Icon(
            widget.navItems2![i],
            color: initialIndex == i ? Colors.black : Colors.black54,
            size: initialIndex == i ? widget.height / 1.8 : widget.height / 2,
          ),
          Text(
            widget.navItems![i],
            style: widget.textStyle != null
                ? widget.textStyle!.copyWith(
              color: initialIndex == i
                  ? widget.activeColor
                  : widget.passiveColor,
              fontSize: initialIndex == i
                  ? widget.textStyle!.fontSize! + 2
                  : widget.textStyle!.fontSize,
            )
                : TextStyle(
              color: initialIndex == i
                  ? widget.activeColor
                  : widget.passiveColor,
              fontSize: initialIndex == i
                  ? widget.height / 5
                  : widget.height / 6,
            ),
          ),
        ],
      );
    } else if (widget.navItems != null) {
      return Text(
        widget.navItems![i],
        style: widget.textStyle != null
            ? widget.textStyle!.copyWith(
          color: initialIndex == i
              ? widget.activeColor
              : widget.passiveColor,
          fontSize: initialIndex == i
              ? widget.textStyle!.fontSize! + 2
              : widget.textStyle!.fontSize,
        )
            : TextStyle(
          color: initialIndex == i
              ? widget.activeColor
              : widget.passiveColor,
          fontSize: initialIndex == i ? 13 : 11,
        ),
      );
    } else
      return Icon(
        widget.navItems2![i],
        color: initialIndex == i ? Colors.black : Colors.black54,
        size: initialIndex == i ? widget.height / 1.8 : widget.height / 2,
      );
  }

  num lengthReturner() {
    if (widget.navItems != null) {
      return widget.navItems!.length;
    } else if (widget.navItems2 != null) {
      return widget.navItems2!.length;
    } else {
      return 2;
    }
  }

  num indicatorSizeFactor() {
    int factor = 1;
    switch (widget.indicatorType) {
      case IndicatorTypes.ULTRASMALL:
        factor = 15;
        break;
      case IndicatorTypes.SMALL:
        factor = 5;
        break;

      case IndicatorTypes.MEDIUM:
        factor = 3;
        break;
      case IndicatorTypes.LARGE:
        factor = 2;
        break;
      case IndicatorTypes.MAX:
        factor = 1;
        break;
    }
    return factor;
  }
}
